@@ -32,6 +32,7 @@
 #include <unistd.h>
 
 #include <gdk/gdk.h>
+#include <cairo/cairo-xlib.h>
 
 #include "display.h"
 #include "screen.h"
@@ -160,6 +161,8 @@ typedef struct _MetaCompWindow
   gboolean damaged;
   gboolean shaped;
 
+  XRectangle shape_bounds;
+
   MetaCompWindowType type;
 
   Damage damage;
@@ -1932,6 +1935,11 @@ add_win (MetaScreen *screen,
   cw->damaged = FALSE;
   cw->shaped = is_shaped (display, xwindow);
 
+  cw->shape_bounds.x = cw->attrs.x;
+  cw->shape_bounds.y = cw->attrs.y;
+  cw->shape_bounds.width = cw->attrs.width;
+  cw->shape_bounds.height = cw->attrs.height;
+
   if (cw->attrs.class == InputOnly)
     cw->damage = None;
   else
@@ -2077,6 +2085,7 @@ resize_win (MetaCompWindow *cw,
   Display *xdisplay = meta_display_get_xdisplay (display);
   MetaCompScreen *info = meta_screen_get_compositor_data (screen);
   XserverRegion damage;
+  XserverRegion shape;
   gboolean debug;
 
   debug = DISPLAY_COMPOSITOR (display)->debug;
@@ -2168,6 +2177,10 @@ resize_win (MetaCompWindow *cw,
       XFixesCopyRegion (xdisplay, damage, cw->extents);
     }
 
+  shape = XFixesCreateRegion (xdisplay, &cw->shape_bounds, 1);
+  XFixesUnionRegion (xdisplay, damage, damage, shape);
+  XFixesDestroyRegion (xdisplay, shape);
+
   dump_xserver_region ("resize_win", display, damage);
   add_damage (screen, damage);
 
@@ -2526,6 +2539,21 @@ process_shape (MetaCompositorXRender *compositor,
 
       if (event->shaped && !cw->shaped)
         cw->shaped = TRUE;
+
+      if (event->shaped == True)
+        {
+          cw->shape_bounds.x = cw->attrs.x + event->x;
+          cw->shape_bounds.y = cw->attrs.y + event->y;
+          cw->shape_bounds.width = event->width;
+          cw->shape_bounds.height = event->height;
+        }
+      else
+        {
+          cw->shape_bounds.x = cw->attrs.x;
+          cw->shape_bounds.y = cw->attrs.y;
+          cw->shape_bounds.width = cw->attrs.width;
+          cw->shape_bounds.height = cw->attrs.height;
+        }
     }
 }
 
@@ -2799,23 +2827,38 @@ xrender_end_move (MetaCompositor *compositor,
 #ifdef HAVE_COMPOSITE_EXTENSIONS
 #endif
 }
+#endif /* 0 */
 
 static void
 xrender_free_window (MetaCompositor *compositor,
                      MetaWindow     *window)
 {
 #ifdef HAVE_COMPOSITE_EXTENSIONS
-  /* FIXME: When an undecorated window is hidden this is called,
-     but the window does not get readded if it is subsequentally shown again
-     See http://bugzilla.gnome.org/show_bug.cgi?id=504876
+  MetaCompositorXRender *xrc;
+  MetaFrame *frame;
+  Window xwindow;
 
-     I don't *think* theres any need for this call anyway, leaving it out
-     does not seem to cause any side effects so far, but I should check with
-     someone who understands more. */
-  /* destroy_win (compositor->display, window->xwindow, FALSE); */
+  xrc = (MetaCompositorXRender *) compositor;
+  frame = meta_window_get_frame (window);
+  xwindow = None;
+
+  if (frame)
+    {
+      xwindow = meta_frame_get_xwindow (frame);
+    }
+  else
+    {
+      /* FIXME: When an undecorated window is hidden this is called, but the
+       * window does not get readded if it is subsequentally shown again. See:
+       * http://bugzilla.gnome.org/show_bug.cgi?id=504876
+       */
+      /* xwindow = meta_window_get_xwindow (window); */
+    }
+
+  if (xwindow != None)
+    destroy_win (xrc->display, xwindow, FALSE);
 #endif
 }
-#endif /* 0 */
 
 static void
 xrender_process_event (MetaCompositor *compositor,
@@ -2890,24 +2933,42 @@ xrender_process_event (MetaCompositor *compositor,
 #endif
 }
 
-static Pixmap
-xrender_get_window_pixmap (MetaCompositor *compositor,
-                           MetaWindow     *window)
+static cairo_surface_t *
+xrender_get_window_surface (MetaCompositor *compositor,
+                            MetaWindow     *window)
 {
 #ifdef HAVE_COMPOSITE_EXTENSIONS
-  MetaCompWindow *cw = NULL;
-  MetaScreen *screen = meta_window_get_screen (window);
-  MetaFrame *frame = meta_window_get_frame (window);
+  MetaFrame *frame;
+  Window xwindow;
+  MetaScreen *screen;
+  MetaCompWindow *cw;
+  MetaCompositorXRender *xrc;
+  Display *display;
+  Pixmap pixmap;
+
+  frame = meta_window_get_frame (window);
+
+  if (frame)
+    xwindow = meta_frame_get_xwindow (frame);
+  else
+    xwindow = meta_window_get_xwindow (window);
+
+  screen = meta_window_get_screen (window);
+  cw = find_window_for_screen (screen, xwindow);
 
-  cw = find_window_for_screen (screen, frame ? meta_frame_get_xwindow (frame) :
-                               meta_window_get_xwindow (window));
   if (cw == NULL)
-    return None;
+    return NULL;
+
+  xrc = (MetaCompositorXRender *) compositor;
+  display = meta_display_get_xdisplay (xrc->display);
 
   if (meta_window_is_shaded (window))
-    return cw->shaded_back_pixmap;
+    pixmap = cw->shaded_back_pixmap;
   else
-    return cw->back_pixmap;
+    pixmap = cw->back_pixmap;
+
+  return cairo_xlib_surface_create (display, pixmap, cw->attrs.visual,
+                                    cw->attrs.width, cw->attrs.height);
 #endif
 }
 
@@ -3093,8 +3154,9 @@ static MetaCompositor comp_info = {
   xrender_remove_window,
   xrender_set_updates,
   xrender_process_event,
-  xrender_get_window_pixmap,
+  xrender_get_window_surface,
   xrender_set_active_window,
+  xrender_free_window,
   xrender_maximize_window,
   xrender_unmaximize_window,
 };
@@ -3103,7 +3165,7 @@ MetaCompositor *
 meta_compositor_xrender_new (MetaDisplay *display)
 {
 #ifdef HAVE_COMPOSITE_EXTENSIONS
-  char *atom_names[] = {
+  const gchar *atom_names[] = {
     "_XROOTPMAP_ID",
     "_XSETROOT_ID",
     "_NET_WM_WINDOW_OPACITY",
@@ -3133,7 +3195,7 @@ meta_compositor_xrender_new (MetaDisplay *display)
   xrc->display = display;
 
   meta_verbose ("Creating %d atoms\n", (int) G_N_ELEMENTS (atom_names));
-  XInternAtoms (xdisplay, atom_names, G_N_ELEMENTS (atom_names),
+  XInternAtoms (xdisplay, (gchar **) atom_names, G_N_ELEMENTS (atom_names),
                 False, atoms);
 
   xrc->atom_x_root_pixmap = atoms[0];
