@@ -116,6 +116,7 @@ typedef struct
   MetaRectangle        orig;
   MetaRectangle        current;
   MetaFrameBorders    *borders;
+  gboolean             must_free_borders;
   ActionType           action_type;
   gboolean             is_user_action;
 
@@ -196,9 +197,11 @@ static void place_window_if_needed       (MetaWindow     *window,
                                           ConstraintInfo *info);
 static void update_onscreen_requirements (MetaWindow     *window,
                                           ConstraintInfo *info);
-static void extend_by_frame              (MetaRectangle           *rect,
+static void extend_by_frame              (MetaWindow              *window,
+                                          MetaRectangle           *rect,
                                           const MetaFrameBorders  *borders);
-static void unextend_by_frame            (MetaRectangle           *rect,
+static void unextend_by_frame            (MetaWindow              *window,
+                                          MetaRectangle           *rect,
                                           const MetaFrameBorders  *borders);
 static inline void get_size_limits       (const MetaWindow        *window,
                                           const MetaFrameBorders  *borders,
@@ -330,7 +333,7 @@ meta_window_constrain (MetaWindow          *window,
    * not gobject-style--gobject would be more pain than it's worth) or
    * smart pointers would be so much nicer here.  *shrug*
    */
-  if (!orig_borders)
+  if (info.must_free_borders)
     g_free (info.borders);
 }
 
@@ -351,9 +354,15 @@ setup_constraint_info (ConstraintInfo      *info,
 
   /* Create a fake frame geometry if none really exists */
   if (orig_borders && !window->fullscreen)
-    info->borders = orig_borders;
+    {
+      info->borders = orig_borders;
+      info->must_free_borders = FALSE;
+    }
   else
-    info->borders = g_new0 (MetaFrameBorders, 1);
+    {
+      info->borders = g_new0 (MetaFrameBorders, 1);
+      info->must_free_borders = TRUE;
+    }
 
   if (flags & META_IS_MOVE_ACTION && flags & META_IS_RESIZE_ACTION)
     info->action_type = ACTION_MOVE_AND_RESIZE;
@@ -535,6 +544,7 @@ place_window_if_needed(MetaWindow     *window,
       info->fixed_directions = FIXED_DIRECTION_NONE;
     }
 
+
   if (window->placed || did_placement)
     {
       if (window->maximize_horizontally_after_placement ||
@@ -579,6 +589,9 @@ place_window_if_needed(MetaWindow     *window,
           window->maximize_horizontally_after_placement = FALSE;
           window->maximize_vertically_after_placement = FALSE;
         }
+
+      window->user_rect = info->current;
+
       if (window->minimize_after_placement)
         meta_window_minimize (window);
     }
@@ -625,7 +638,7 @@ update_onscreen_requirements (MetaWindow     *window,
   /* The require onscreen/on-single-xinerama and titlebar_visible
    * stuff is relative to the outer window, not the inner
    */
-  extend_by_frame (&info->current, info->borders);
+  extend_by_frame (window, &info->current, info->borders);
 
   /* Update whether we want future constraint runs to require the
    * window to be on fully onscreen.
@@ -660,8 +673,8 @@ update_onscreen_requirements (MetaWindow     *window,
     {
       MetaRectangle titlebar_rect;
 
-      titlebar_rect = info->current;
-      titlebar_rect.height = info->borders->visible.top;
+      meta_window_get_titlebar_rect (window, &titlebar_rect);
+
       old = window->require_titlebar_visible;
       window->require_titlebar_visible =
         meta_rectangle_overlaps_with_region (info->usable_screen_region,
@@ -674,27 +687,49 @@ update_onscreen_requirements (MetaWindow     *window,
     }
 
   /* Don't forget to restore the position of the window */
-  unextend_by_frame (&info->current, info->borders);
+  unextend_by_frame (window, &info->current, info->borders);
 }
 
 static void
-extend_by_frame (MetaRectangle           *rect,
+extend_by_frame (MetaWindow             *window,
+                 MetaRectangle          *rect,
                  const MetaFrameBorders *borders)
 {
-  rect->x -= borders->visible.left;
-  rect->y -= borders->visible.top;
-  rect->width  += borders->visible.left + borders->visible.right;
-  rect->height += borders->visible.top + borders->visible.bottom;
+  if (window->frame)
+    {
+      rect->x -= borders->visible.left;
+      rect->y -= borders->visible.top;
+      rect->width  += borders->visible.left + borders->visible.right;
+      rect->height += borders->visible.top + borders->visible.bottom;
+    }
+  else
+    {
+      rect->x += window->custom_frame_extents.left;
+      rect->y += window->custom_frame_extents.top;
+      rect->width -= window->custom_frame_extents.left + window->custom_frame_extents.right;
+      rect->height -= window->custom_frame_extents.top + window->custom_frame_extents.bottom;
+    }
 }
 
 static void
-unextend_by_frame (MetaRectangle           *rect,
+unextend_by_frame (MetaWindow             *window,
+                   MetaRectangle          *rect,
                    const MetaFrameBorders *borders)
 {
-  rect->x += borders->visible.left;
-  rect->y += borders->visible.top;
-  rect->width  -= borders->visible.left + borders->visible.right;
-  rect->height -= borders->visible.top + borders->visible.bottom;
+  if (window->frame)
+    {
+      rect->x += borders->visible.left;
+      rect->y += borders->visible.top;
+      rect->width  -= borders->visible.left + borders->visible.right;
+      rect->height -= borders->visible.top + borders->visible.bottom;
+    }
+  else
+    {
+      rect->x -= window->custom_frame_extents.left;
+      rect->y -= window->custom_frame_extents.top;
+      rect->width += window->custom_frame_extents.left + window->custom_frame_extents.right;
+      rect->height += window->custom_frame_extents.top + window->custom_frame_extents.bottom;
+    }
 }
 
 static inline void
@@ -714,23 +749,41 @@ get_size_limits (const MetaWindow        *window,
 
   if (include_frame)
     {
-      int fw = borders->visible.left + borders->visible.right;
-      int fh = borders->visible.top + borders->visible.bottom;
+      int fw;
+      int fh;
 
-      min_size->width  += fw;
-      min_size->height += fh;
-      /* Do check to avoid overflow (e.g. max_size->width & max_size->height
-       * may be set to G_MAXINT by meta_set_normal_hints()).
-       */
-      if (max_size->width < (G_MAXINT - fw))
-        max_size->width += fw;
-      else
-        max_size->width = G_MAXINT;
+      if (window->frame)
+        {
+          fw = borders->visible.left + borders->visible.right;
+          fh = borders->visible.top + borders->visible.bottom;
 
-      if (max_size->height < (G_MAXINT - fh))
-        max_size->height += fh;
+          min_size->width += fw;
+          min_size->height += fh;
+
+          /* Do check to avoid overflow (e.g. max_size->width & max_size->height
+           * may be set to G_MAXINT by meta_set_normal_hints()).
+           */
+          if (max_size->width < (G_MAXINT - fw))
+            max_size->width += fw;
+          else
+            max_size->width = G_MAXINT;
+
+          if (max_size->height < (G_MAXINT - fh))
+            max_size->height += fh;
+          else
+            max_size->height = G_MAXINT;
+        }
       else
-        max_size->height = G_MAXINT;
+        {
+          fw = window->custom_frame_extents.left + window->custom_frame_extents.right;
+          fh = window->custom_frame_extents.top + window->custom_frame_extents.bottom;
+
+          min_size->width -= fw;
+          min_size->height -= fh;
+
+          max_size->width -= fw;
+          max_size->height -= fh;
+        }
     }
 }
 
@@ -759,7 +812,7 @@ constrain_modal_dialog (MetaWindow         *window,
       y += parent->frame->rect.y;
 
       meta_frame_calc_borders (parent->frame, &borders);
-      y += borders.visible.top;
+      y += borders.total.top;
 
       y += info->borders->visible.top;
    }
@@ -820,14 +873,14 @@ constrain_maximization (MetaWindow         *window,
       active_workspace_struts = window->screen->active_workspace->all_struts;
 
       target_size = info->current;
-      extend_by_frame (&target_size, info->borders);
+      extend_by_frame (window, &target_size, info->borders);
       meta_rectangle_expand_to_avoiding_struts (&target_size,
                                                 &info->entire_xinerama,
                                                 direction,
                                                 active_workspace_struts);
    }
   /* Now make target_size = maximized size of client window */
-  unextend_by_frame (&target_size, info->borders);
+  unextend_by_frame (window, &target_size, info->borders);
 
   /* Check min size constraints; max size constraints are ignored for maximized
    * windows, as per bug 327543.
@@ -887,7 +940,7 @@ constrain_tiling (MetaWindow *window,
    * use an external function for the actual calculation
    */
   meta_window_get_current_tile_area (window, &target_size);
-  unextend_by_frame (&target_size, info->borders);
+  unextend_by_frame (window, &target_size, info->borders);
 
   /* Check min size constraints; max size constraints are ignored as for
    * maximized windows.
@@ -1236,7 +1289,7 @@ do_screen_and_xinerama_relative_constraints (
   /* Determine whether constraint applies; exit if it doesn't */
   how_far_it_can_be_smushed = info->current;
   get_size_limits (window, info->borders, TRUE, &min_size, &max_size);
-  extend_by_frame (&info->current, info->borders);
+  extend_by_frame (window, &info->current, info->borders);
 
   if (info->action_type != ACTION_MOVE)
     {
@@ -1256,7 +1309,7 @@ do_screen_and_xinerama_relative_constraints (
                                         &info->current);
   if (exit_early || constraint_satisfied || check_only)
     {
-      unextend_by_frame (&info->current, info->borders);
+      unextend_by_frame (window, &info->current, info->borders);
       return constraint_satisfied;
     }
 
@@ -1280,7 +1333,7 @@ do_screen_and_xinerama_relative_constraints (
                                       info->fixed_directions,
                                       &info->current);
 
-  unextend_by_frame (&info->current, info->borders);
+  unextend_by_frame (window, &info->current, info->borders);
   return TRUE;
 }
 
