@@ -39,6 +39,7 @@
 #include "stack.h"
 #include "iconcache.h"
 #include <X11/Xutil.h>
+#include <cairo.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 
 typedef struct _MetaGroup MetaGroup;
@@ -315,15 +316,16 @@ struct _MetaWindow
   /* if TRUE, application is buggy and SYNC resizing is turned off */
   guint disable_sync : 1;
 
+  /* if non-NULL, the bounds of the window frame */
+  cairo_region_t *frame_bounds;
+
   /* Note: can be NULL */
   GSList *struts;
 
-#ifdef HAVE_XSYNC
   /* XSync update counter */
   XSyncCounter sync_request_counter;
   guint sync_request_serial;
   GTimeVal sync_request_time;
-#endif
 
   /* Number of UnmapNotify that are caused by us, if
    * we get UnmapNotify with none pending then the client
@@ -519,6 +521,8 @@ void        meta_window_get_geometry         (MetaWindow  *window,
                                               int         *y,
                                               int         *width,
                                               int         *height);
+void        meta_window_get_input_rect       (const MetaWindow *window,
+                                              MetaRectangle    *rect);
 void        meta_window_get_outer_rect       (const MetaWindow *window,
                                               MetaRectangle    *rect);
 void        meta_window_get_xor_rect         (MetaWindow          *window,
@@ -662,4 +666,7 @@ void meta_window_update_icon_now (MetaWindow *window);
 gboolean meta_window_can_tile_side_by_side (MetaWindow *window);
 gboolean meta_window_is_client_decorated (MetaWindow *window);
 
+void meta_window_get_titlebar_rect (MetaWindow    *window,
+                                    MetaRectangle *titlebar_rect);
+
 #endif
