@@ -664,6 +664,26 @@ dock_cb (GSimpleAction *action,
     }
 }
 
+static void
+override_background_color (GtkWidget *widget,
+                           GdkRGBA   *rgba)
+{
+  gchar          *css;
+  GtkCssProvider *provider;
+
+  provider = gtk_css_provider_new ();
+
+  css = g_strdup_printf ("* { background-color: %s; }",
+                         gdk_rgba_to_string (rgba));
+  gtk_css_provider_load_from_data (provider, css, -1, NULL);
+  g_free (css);
+
+  gtk_style_context_add_provider (gtk_widget_get_style_context (widget),
+                                  GTK_STYLE_PROVIDER (provider),
+                                  GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+  g_object_unref (provider);
+}
+
 static void
 desktop_cb (GSimpleAction *action,
             GVariant      *parameter,
@@ -685,7 +705,7 @@ desktop_cb (GSimpleAction *action,
   desktop_color.blue = 0.65;
   desktop_color.alpha = 1.0;
 
-  gtk_widget_override_background_color (window, 0, &desktop_color);
+  override_background_color (window, &desktop_color);
 
   label = focus_label (window);
 
